unit uGooglePageRank;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, IdBaseComponent, IdComponent, IdTCPConnection,
  IdTCPClient, WinInet;

type

  TPageRankCalc = class
  private
  protected
    class function PageRankStrToNum(const S: string; Check: Int64; Magic: Cardinal): Int64;
    class function PageRankHashURL(const S: string): Int64;
    class function CheckHash(HashNum: Int64): AnsiString;
  public
    class function SearchURI(const url: AnsiString): AnsiString;
  end;

function GetGooglePageRank(url: AnsiString; timeout: Integer): AnsiString;

implementation

var
  httpAppDescription: AnsiString = 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 1.1.4322';

function httpExec(sAppName, sMethod, sHost, sRemoteFile, sHeader: AnsiString;
  msData: TMemoryStream; timeout: Integer): AnsiString;
var
  i               : Integer;
  hSession,
  hOpenReq,
  hConnection     : HInternet;
  buf             : array[0..4095] of char;
  bufLen, dum     : DWORD;
  sHeaderLine     : AnsiString;
  sl              : TStringList;
begin
  Result := '';
  dum := 0;
  if sAppName = '' then
    sAppName := httpAppDescription;
  hSession := InternetOpen(PChar(sAppName), INTERNET_OPEN_TYPE_PRECONFIG, nil, nil, 0);
  try
    if hSession = nil then
      Result := 'Error openning internet'
    else
    begin
      InternetSetOption(hSession, INTERNET_OPTION_RECEIVE_TIMEOUT, @timeout, sizeof(timeout));
      hConnection := InternetConnect(hSession, PAnsiChar(sHost), INTERNET_DEFAULT_HTTP_PORT, nil, nil, INTERNET_SERVICE_HTTP, 0, 0);
      try
        if hConnection = nil then
          Result := 'Error openning internet connection'
        else
        begin
          InternetSetOption(hConnection, INTERNET_OPTION_RECEIVE_TIMEOUT, @timeout, sizeof(timeout));
          hOpenReq := HTTPOpenRequest(hConnection, PAnsiChar(sMethod), PAnsiChar(sRemoteFile), 'HTTP/1.1', nil, nil,
            INTERNET_FLAG_SECURE or INTERNET_FLAG_KEEP_CONNECTION{This is necessary if authentication is required.}, 0);
          try
            if hOpenReq = nil then
              Result := 'Error openning internet request'
            else
            begin
              sl := TStringList.Create;
              try
                sl.Text := sHeader;
                for i := 0 to sl.Count - 1 do
                begin
                  sHeaderLine := sl[i];
                  HttpAddRequestHeaders(hOpenReq, PChar(sHeaderLine), Length(sHeaderLine), HTTP_ADDREQ_FLAG_ADD);
                end;
              finally
                sl.Free;
              end;
              InternetSetOption(hOpenReq, INTERNET_OPTION_RECEIVE_TIMEOUT, @timeout, sizeof(timeout));
              if Assigned(msData) then
                HTTPSendRequest(hOpenReq, nil, 0, msData.memory, msData.Size)
              else
                HTTPSendRequest(hOpenReq, nil, 0, nil, 0);
              HTTPQueryInfo(hOpenReq, HTTP_QUERY_RAW_HEADERS, @buf, bufLen, dum);
              Result := AnsiString(buf);
            end;
          finally
            InternetCloseHandle(hOpenReq);
          end;
        end;
      finally
        InternetCloseHandle(hConnection);
      end;
    end;
  finally
    InternetCloseHandle(hSession);
  end;
end;


function UrlEncodeRFC3986(S: string): string;

  function _IntToHex(Value: Integer; Digits: Integer): String;
  begin
    Result := SysUtils.IntToHex(Value, Digits);
  end;

var
  I: Integer;
  Ch: Char;
begin
  Result := '';
  for I := 1 to Length(S) do begin
    Ch := S[I];
    if ((Ch >= '0') and (Ch <= '9')) or
    ((Ch >= 'a') and (Ch <= 'z')) or
    ((Ch >= 'A') and (Ch <= 'Z')) or
    (Ch = '.') or (Ch = '-') or (Ch = '_') or (Ch = '~')then
      Result := Result + Ch
    else
      Result := Result + '%' + _IntToHex(Ord(Ch), 2);
  end;
end;

class function TPageRankCalc.CheckHash(HashNum: Int64): AnsiString;
var
  CheckByte: Int64;
  Flag: Integer;
  HashStr: AnsiString;
  Len: Integer;
  I: Integer;
  Re: Byte;
begin
  CheckByte := 0;
  Flag := 0;
  HashStr := Format('%d', [HashNum]);
  Len := Length(HashStr);
  for I := Len downto 1 do
  begin
    Re := StrToInt(HashStr[I]);
    if (Flag mod 2) = 1 then
    begin
      Re := Re + Re;
      Re := (Re div 10) + (Re mod 10);
    end;
    CheckByte := CheckByte + Re;
    Inc(Flag);
  end;
  CheckByte := CheckByte mod 10;
  if (CheckByte <> 0) then
  begin
    CheckByte := 10 - CheckByte;
    if (Flag mod 2) = 1 then
    begin
      if (CheckByte mod 2) = 1 then
        CheckByte := CheckByte + 9;
      CheckByte := CheckByte shr 1;
    end;
  end;
  Result := '7' + IntToStr(CheckByte) + HashStr;
end;

class function TPageRankCalc.PageRankHashURL(const S: string): Int64;
var
  Check1, Check2: Int64;
  T1, T2: Int64;
begin
  Check1 := PageRankStrToNum(S, $1505, $21);
  Check2 := PageRankStrToNum(S, $0, $1003F);
  //Form2.Label5.Caption := FormatBin(Check1);
  //Form2.Label8.Caption := FormatBin(Check2);
  Check1 := Check1 shr 2;
  //Form2.Label6.Caption := FormatBin(Check1);
  Check1 := ((Check1 shr 4) and $3FFFFC0) or (Check1 and $3F);
  Check1 := ((Check1 shr 4) and $3FFC00) or (Check1 and $3FF);
  Check1 := ((Check1 shr 4) and $3C000) or (Check1 and $3FFF);
  T1 := ((((Check1 and $3C0) shl 4) or (Check1 and $3C)) shl 2) or (Check2 and $F0F);
  T2 := ((((Check1 and $FFFFC000) shl 4) or (Check1 and $3C00)) shl $A) or (Check2 and $F0F0000);
  Result := T1 or T2;
end;

class function TPageRankCalc.PageRankStrToNum(const S: string; Check: Int64; Magic: Cardinal): Int64;
const
  Int32Uint = 4294967296;
var
  _length: integer;
  I: Integer;
  i2147483648: Int64;
begin
  Result := Check;
  _length := Length(S);
  for I := 1 to _length do
  begin
    Result := Result * Magic;
    if (Result >= Int32Uint) then
    begin
      Result := Result - Int32Uint * Integer(Result div Int32UInt);  //should be div?
      i2147483648 := -2147483647;
      Dec(i2147483648);
      if Result < i2147483648 then
        Result := Result + Int32UInt;
    end;
    Result := Result + Ord(S[I]);
  end;
end;

class function TPageRankCalc.SearchURI(const url: AnsiString): AnsiString;
begin
  //Result := '/search?client=navclient-auto&ch=' + CheckHash(PageRankHashURL(url)) + '&features=Rank&q=info:'+UrlEncodeRFC3986(url)+'&num=100&filter=0';
  Result := '/tbr?client=navclient-auto&ch=' + CheckHash(PageRankHashURL(url)) + '&features=Rank&q=info:'+UrlEncodeRFC3986(url)+'&num=100&filter=0';
end;

function GetGooglePageRank(url: AnsiString; timeout: Integer): AnsiString;
var
  Msg: AnsiString;
  Rsp: TStringList;
  S, Response, location: string;
  PIni: Integer;
  sPR: string;
  IdTCPClient1: TIdTCPClient;
begin
  IdTCPClient1 := TIdTCPClient.Create(nil);
  IdTCPClient1.Host := 'toolbarqueries.google.com';
  IdTCPClient1.Port := 80;
  Msg := '';
  Rsp := TStringList.Create;
  try
    location := TPageRankCalc.SearchURI(url);
    Msg := Msg + Format('GET %s  HTTP/1.1', [location]) + #13#10;
    Msg := Msg + 'Host: toolbarqueries.google.com' +  #13#10;
    Msg := Msg + 'User-Agent: Mozilla/4.0 (compatible; GoogleToolbar 2.0.114-big; Windows XP 5.1)' + #13#10;
    Msg := Msg + 'Connection: Close'  + #13#10;
    Msg := Msg + '' + #13#10;  //header end
    Response := httpExec('', 'GET', 'toolbarqueries.google.com', location, Msg, nil, timeout);
    Rsp.Text := Response;
    {IdTCPClient1.Connect;
    try
      IdTCPClient1.IOHandler.WriteDirect(TBytes(@Msg[1]), Length(Msg));
      try
        repeat
          s := IdTCPClient1.IOHandler.ReadLn();
          if IdTCPClient1.IOHandler.ReadLnTimedout then
            S := '';
          Rsp.Add(s);
          IdTCPClient1.IOHandler.ReadStrings(Rsp);
        until false;
      except
        on EIdConnClosedGracefully do
          IdTCPClient1.Disconnect;
      end;}
      sPR := 'Error';
      if Rsp[0]='HTTP/1.1 200 OK' then
      begin
        PIni := Pos('Rank_', Rsp[Rsp.Count - 1]);
        if PIni <> 0 then
          sPR := Copy(Rsp[Rsp.Count - 1], PIni + 9, MaxInt);
      end;
      //ShowMessage('Page rank is: ' + sPR);
      Result := sPR;
    {finally
      if IdTCPClient1.Connected then
        IdTCPClient1.Disconnect;
    end;}
  finally
    Rsp.Free;
    IdTCPClient1.Free;
  end;
end;

end.
