unit uHelper;

interface

uses
  Windows, Classes, SysUtils, OleCtrls, SHDocVw, WinInet, Forms, Dialogs, StdCtrls, ADODB, StrUtils, ShellApi, Winsock, RegExpr, Registry;

function ASCIIURLDecode(Link: string):string;
function  NameToIP(host: string):string;
procedure RemoveScripts(var DataBuf: string);
procedure RemoveHead(var DataBuf: string);
//function  ExtractParagraph(var DataBuf: string; var Paragraph: string; sTag, eTag: string):boolean;
function  ExtractParagraph(var DataBuf: string; var Paragraph: string; sTag, eTag: string; sTag2: string = ''):boolean;
function  ExtractParagraphJSON(var DataBuf: string; var Paragraph: string; sTag, eTag: string):boolean;
function  ExtractSParagraphJSON(var DataBuf: string; var Paragraph: string; sTag, eTag: string;  var FOffset: Integer):boolean;
function  ExtractParagraph2(var DataBuf: string; var Paragraph: string; sTag, eTag: string):boolean;
function  ExtractLink(text: string; sTag, eTag: string):string;
function  ExtractText(text: string; sTag, eTag: string):string;
procedure BrowseUrl(url: string; web: TWebBrowser);
function  OpenURL(const URL: string) : boolean;
procedure DownloadFile(const url: string; var stream: TStringStream);
//function  DownloadFile2(const url: string): string;
function  DownloadFile3(const url: string): string;
function  DownloadFile4(const url: string; timeout: integer): string;
function  ExtractLocation(webUrl: string):string;
function  GetDBFileName:string; // return the file name and path to the database file; useful in case we change it later
function  PopulateCB(table_name, filter_column_name, setting_name, res_column_name: string; cb: TComboBox):integer;
function  GetGooglePageRank(aURL: string):string;
function  PathToFolders(path: string; var folders: TStringList):integer;
function  CheckPath(folders: TStringList; create: boolean):boolean;
function  CheckFolder(path: string; create: boolean):boolean;
function  NormalizeFilename(path: string):string;
function  NormalizeFilename2(path: string):string;
function  DelTree(DirName: string): boolean;
function  GetPageDescription(html: string):string;
function  GetPageKeywords(html: string):string;
function  GetGooglePageRank2(aURL: string; var test: string):string;
function  ExtractDomain(webUrl: string):string;
function  GetAnchorText(html, url: string):string;
procedure GetLinks(text: string; domain: string; var sl: TStringList);
function  GetIndexedPages(url: string):longint;
function  GetOutboundLinksCount(page, url: string):integer;
function  GetAllLinksCount(page: string):integer;
function  UrlEncodeRFC3986(S: string): string;
function  Say(what: string):boolean;

implementation

uses FastStrings, FastStringFuncs;

function OpenURL(const URL: string) : boolean;
var
   Browser: string;
begin
   Result := True;
   Browser := '';
   with TRegistry.Create do
   try
     RootKey := HKEY_CLASSES_ROOT;
Access := KEY_QUERY_VALUE;
     if OpenKey('\htmlfile\shell\open\command', False) then
       Browser := ReadString('') ;
     CloseKey;
   finally
     Free;
   end;
   if Browser = '' then
   begin
     Result := False;
     Exit;
   end;
   Browser := Copy(Browser, Pos('"', Browser) + 1, Length(Browser)) ;
   Browser := Copy(Browser, 1, Pos('"', Browser) - 1) ;
   ShellExecute(0, 'open', PChar(Browser), PChar(URL), nil, SW_SHOW) ;
end;

function NameToIP(host: string):string;
type
  TAPInAddr = Array[0..100] of PInAddr;
  PAPInAddr = ^TAPInAddr;
var
  WSAData: TWSAData;
  HostEntPtr: PHostEnt;
  pptr: PAPInAddr;
begin
  Result := '';
  host := Trim(host);
  if (host = '') then Exit;
  // WinSock initialisation
  WSAStartUp($101, WSAData);
  try
    // Get host entry by hostname
    HostEntPtr := GetHostByName(PChar(host));
    if HostEntPtr = nil then
    begin
      WSACleanUp(); // uninitialise WinSock
      Exit;
    end;
    // Make list of host IPs
    pptr := PAPInAddr(HostEntPtr^.h_addr_list);
    if pptr^[0] <> nil then Result := inet_ntoa(pptr^[0]^);
  except
  end;
  WSACleanUp(); // uninitialise WinSock
end;

function GetPageDescription(html: string):string;
var
  stop: boolean;
  t:    string;
begin
  Result := '';

  stop := true;
  repeat
    stop := ExtractParagraph(html, t, '<meta ', '>');
    if (Trim(t) <> '') then
    begin
      if (Pos('description', LowerCase(t)) > 0) then
      begin
        Result := ExtractText(t, 'content="', '"');
        Break;
      end;
    end;
  until not stop;
  if (Trim(Result) = '') then Result := 'No description';
end;

function GetPageKeywords(html: string):string;
var
  stop: boolean;
  t:    string;
begin
  Result := '';

  stop := true;
  repeat
    stop := ExtractParagraph(html, t, '<meta ', '>');
    if (Trim(t) <> '') then
    begin
      if (Pos('keywords', LowerCase(t)) > 0) then
      begin
        Result := ExtractText(t, 'content="', '"');
        Break;
      end;
    end;
  until not stop;
  if (Trim(Result) = '') then Result := '';
end;

procedure RemoveScripts(var DataBuf: string);
var
  iStart, iStop: integer;
  stop: boolean;
begin
  // start removing any script inside the page
  stop := False;
  repeat
    iStart := Pos('<script', DataBuf);
    if iStart <> 0 then
    begin
      iStop := Pos('</script>', DataBuf) + Length('</script>');
      DataBuf := Copy(DataBuf, 0, iStart - 1) + Copy(DataBuf, iStop, MaxInt);
    end
    else
      stop := True;
  until stop;
end;

procedure RemoveHead(var DataBuf: string);
var
  iStart, iStop: integer;
  stop: boolean;
begin
  stop := False;
  repeat
    iStart := Pos('<head', DataBuf);
    if iStart <> 0 then
    begin
      iStop := Pos('</head>', DataBuf) + Length('</head>');
      DataBuf := Copy(DataBuf, 0, iStart - 1) + Copy(DataBuf, iStop, MaxInt);
    end
    else
      stop := True;
  until stop;
end;

{
function ExtractParagraph(var DataBuf: string; var Paragraph: string; sTag, eTag: string):boolean;
var
  iStart, iStop: integer;
begin
  iStart := Pos(LowerCase(sTag), LowerCase(DataBuf));
  if iStart <> 0 then
  begin
    DataBuf := Copy(DataBuf, iStart, MaxInt);
    iStop := Pos(LowerCase(eTag), LowerCase(DataBuf)) + Length(eTag);
    Paragraph := Copy(DataBuf, 0, iStop - 1);
    DataBuf := Copy(DataBuf, iStop, MaxInt);
    Result    := True;
  end
  else
  begin
    Paragraph := '';
    DataBuf   := '';
    Result    := False;
  end;
end;
}

function  ExtractParagraph(var DataBuf: string; var Paragraph: string; sTag, eTag: string; sTag2: string = ''):boolean;
var
  iStart, iStop, tmp: integer;
begin
  iStart := Pos(LowerCase(sTag), LowerCase(DataBuf));
  if sTag2 <> '' then
  begin
    tmp := Pos(LowerCase(sTag2), LowerCase(DataBuf));
    // Say(Format('-------- tmp = %d', [ tmp ]));
    if (tmp < iStart) and (tmp > 0) then
    begin
      iStart := tmp;
      // Say('------------- tmp < iStart');
    end;
  end;

  //Say(Format('---- iStart = %d | sTag = %s | eTag = %s | sTag2 = %s', [iStart, sTag, eTag, sTag2]));

  if iStart > 0 then
  begin
    DataBuf := Copy(DataBuf, iStart, MaxInt);
    iStop := Pos(LowerCase(eTag), LowerCase(DataBuf)) + Length(eTag);
    //Say(Format('---- iStop = %d', [iStop]));
    Paragraph := Copy(DataBuf, 0, iStop - 1);
    DataBuf := Copy(DataBuf, iStop, MaxInt);
    Result    := True;
  end
  else
  begin
    //Say('---------- iStart <= 0');
    Paragraph := '';
    DataBuf   := '';
    Result    := False;
  end;
end;

function  ExtractParagraphJSON(var DataBuf: string; var Paragraph: string; sTag, eTag: string):boolean;
var
  iStart, iStop: integer;
  tmp: Int64;
begin
  result := false;
  iStart := Pos(sTag, DataBuf);

  if iStart <= 0 then
    exit;
  iStart := iStart + Length(sTag);
  iStop := PosEx(eTag, DataBuf,iStart);
  if iStop <= 0 then
    exit;
  Paragraph := Copy(DataBuf, iStart, iStop-iStart);
  Result    := TryStrToInt64(Paragraph,tmp);
end;

function  ExtractSParagraphJSON(var DataBuf: string; var Paragraph: string; sTag, eTag: string; var FOffset: Integer):boolean;
var
  iStart, iStop: integer;
begin
  result := false;
  iStart := PosEx(sTag, DataBuf,FOffset);

  if iStart <= 0 then
    exit;
  iStart := iStart + Length(sTag);
  iStop := PosEx(eTag, DataBuf,iStart);
  if iStop <= 0 then
    exit;
  Paragraph := Copy(DataBuf, iStart, iStop-iStart);
  Result    := Length(Paragraph)>0;
  FOffset :=  iStop;
end;

function ExtractParagraph2(var DataBuf: string; var Paragraph: string; sTag, eTag: string):boolean;
var
  iStart, iStop: integer;
begin
  iStart := Pos(LowerCase(sTag), LowerCase(DataBuf));
  if iStart <> 0 then
  begin
    DataBuf := Copy(DataBuf, iStart + Length(sTag), MaxInt);
    iStop := Pos(LowerCase(eTag), LowerCase(DataBuf));
    Paragraph := Copy(DataBuf, 1, iStop - 1);
    DataBuf := Copy(DataBuf, iStop, MaxInt);
    Result    := True;
  end
  else
  begin
    Paragraph := '';
    DataBuf   := '';
    Result    := False;
  end;
end;

function ExtractLink(text: string; sTag, eTag: string):string;
var
  iStart, iStop:  integer;
  tmp:            string;
begin
  // Say(Format('TEXT: %s', [text]));
  iStart := Pos(LowerCase(sTag), LowerCase(text)) + Length(sTag);
  tmp := Copy(text, iStart, Length(text));
  iStop  := Pos(LowerCase(eTag), LowerCase(tmp)) + iStart - 1;
  // Say(Format('+++ iStart = %d | iStop = %d', [iStart, iStop]));
  if (iStart > 0) and (iStart < iStop) then
    Result := Trim(Copy(text, iStart, iStop - iStart))
  else
    Result := '';
end;

//
// extract text between two tags
//
function ExtractText(text: string; sTag, eTag: string):string;
var
  iStart, iStop: integer;
begin
  iStart := Pos(LowerCase(sTag), LowerCase(text)) + Length(sTag);
  Delete(text, 1, iStart - 1);
  iStop  := Pos(LowerCase(eTag), LowerCase(text));
  if (iStop > 0) then
    Result := Trim(Copy(text, 1, iStop - 1))
  else
    Result := '';
end;

procedure BrowseUrl(url: string; web: TWebBrowser);
var
  Flags, Headers, TargetFrameName, PostData: OLEVariant;
begin
  Flags := '1';
  TargetFrameName := '';
  PostData := '';
  web.Navigate(Url, Flags, TargetFrameName, PostData, Headers);
end;

procedure DownloadFile(const url: string; var stream: TStringStream);
var
  NetHandle:  HINTERNET;
  UrlHandle:  HINTERNET;
  Buffer:     array[0..1024] of Char;
  BytesRead:  dWord;
  timeout:    longint;
begin
  NetHandle := InternetOpen('Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 1.1.4322)', INTERNET_OPEN_TYPE_PRECONFIG, nil, nil, 0);

  if Assigned(NetHandle) then
  begin
    timeout := 10000; // timeout in 10 seconds
    InternetSetOption(NetHandle, INTERNET_OPTION_RECEIVE_TIMEOUT, @timeout, sizeof(timeout));

    UrlHandle := InternetOpenUrl(NetHandle, PChar(Url), nil, 0, INTERNET_FLAG_RELOAD, 0);

    if Assigned(UrlHandle) then
      { UrlHandle valid? Proceed with download }
    begin
      FillChar(Buffer, SizeOf(Buffer), 0);
      repeat
        FillChar(Buffer, SizeOf(Buffer), 0);
        InternetReadFile(UrlHandle, @Buffer, SizeOf(Buffer), BytesRead);
        stream.Write(Buffer, Bytesread);
        Application.ProcessMessages;
      until BytesRead = 0;
      InternetCloseHandle(UrlHandle);
    end
    else
      { UrlHandle is not valid. Raise an exception. }
      raise Exception.CreateFmt('Cannot open URL %s', [Url]);

    InternetCloseHandle(NetHandle);
  end
  else
    { NetHandle is not valid. Raise an exception }
    raise Exception.Create('Unable to initialize Wininet');
end;

function DownloadFile2(const url: string): string;
var
  NetHandle:  HINTERNET;
  UrlHandle:  HINTERNET;
  Buffer:     array[0..1024] of Char;
  BytesRead:  dWord;
  timeout:    longint;
begin
  Result := '';
  NetHandle := InternetOpen('Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 1.1.4322)', INTERNET_OPEN_TYPE_PRECONFIG, nil, nil, 0);

  if Assigned(NetHandle) then
  begin
    timeout := 10000; // timeout in 10 seconds
    InternetSetOption(NetHandle, INTERNET_OPTION_RECEIVE_TIMEOUT, @timeout, sizeof(timeout));
  
    UrlHandle := InternetOpenUrl(NetHandle, PChar(Url), nil, 0, INTERNET_FLAG_RELOAD, 0);

    if Assigned(UrlHandle) then
      { UrlHandle valid? Proceed with download }
    begin
      FillChar(Buffer, SizeOf(Buffer), 0);
      repeat
        Result := Result + Buffer;
        FillChar(Buffer, SizeOf(Buffer), 0);
        InternetReadFile(UrlHandle, @Buffer, SizeOf(Buffer), BytesRead);
        Application.ProcessMessages;
      until BytesRead = 0;
      InternetCloseHandle(UrlHandle);
    end
    else
      { UrlHandle is not valid. Raise an exception. }
      raise Exception.CreateFmt('Cannot open URL %s', [Url]);

    InternetCloseHandle(NetHandle);
  end
  else
    { NetHandle is not valid. Raise an exception }
    raise Exception.Create('Unable to initialize Wininet');
end;

//
// download a file to a string, do not raise exceptions (if the page can't be downloaded return '')
//
function DownloadFile3(const url: string): string;
var
  NetHandle:  HINTERNET;
  UrlHandle:  HINTERNET;
  Buffer:     array[0..1024] of Char;
  BytesRead:  dWord;
  timeout:    longint;
begin
  Result := '';
  // NetHandle := InternetOpen('Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 1.1.4322)', INTERNET_OPEN_TYPE_PRECONFIG, nil, nil, 0);
  NetHandle := InternetOpen('Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.24) Gecko/20111103 Firefox/3.6.24 ( .NET CLR 3.5.30729; .NET4.0C)', INTERNET_OPEN_TYPE_PRECONFIG, nil, nil, 0);

  if Assigned(NetHandle) then
  begin
    timeout := 10000; // timeout in 10 seconds
    InternetSetOption(NetHandle, INTERNET_OPTION_RECEIVE_TIMEOUT, @timeout, sizeof(timeout));

    UrlHandle := InternetOpenUrl(NetHandle, PChar(Url), nil, 0, INTERNET_FLAG_RELOAD, 0);

    if Assigned(UrlHandle) then // UrlHandle valid? then proceed with download
    begin
      FillChar(Buffer, SizeOf(Buffer), 0);
      repeat
        Result := Result + Buffer;
        FillChar(Buffer, SizeOf(Buffer), 0);
        InternetReadFile(UrlHandle, @Buffer, SizeOf(Buffer), BytesRead);
      until BytesRead = 0;
      InternetCloseHandle(UrlHandle);
    end
    else  // UrlHandle is not valid -> raise an exception
      //raise Exception.CreateFmt('Cannot open URL %s', [Url]);
      Result := '';

    InternetCloseHandle(NetHandle);
  end
  else // NetHandle is not valid -> raise an exception
    //raise Exception.Create('Unable to initialize Wininet!');
    Result := '';
end;

//
// download a file to a string, do not raise exceptions (if the page can't be downloaded return ''); set timeout to 'timeout'
//
function DownloadFile4(const url: string; timeout: integer): string;
var
  NetHandle:  HINTERNET;
  UrlHandle:  HINTERNET;
  Buffer:     array[0..1024] of Char;
  BytesRead:  dWord;
begin
  Result := '';
  NetHandle := InternetOpen('Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 1.1.4322)', INTERNET_OPEN_TYPE_PRECONFIG, nil, nil, 0);
  if Assigned(NetHandle) then
  begin
    InternetSetOption(NetHandle, INTERNET_OPTION_RECEIVE_TIMEOUT, @timeout, sizeof(timeout));

    UrlHandle := InternetOpenUrl(NetHandle, PChar(Url), nil, 0, INTERNET_FLAG_RELOAD, 0);

    if Assigned(UrlHandle) then // UrlHandle valid? then proceed with download
    begin
      FillChar(Buffer, SizeOf(Buffer), 0);
      repeat
        Result := Result + Buffer;
        FillChar(Buffer, SizeOf(Buffer), 0);
        InternetReadFile(UrlHandle, @Buffer, SizeOf(Buffer), BytesRead);
      until BytesRead = 0;
      InternetCloseHandle(UrlHandle);
    end
    else  // UrlHandle is not valid -> raise an exception
      //raise Exception.CreateFmt('Cannot open URL %s', [Url]);
      Result := '';

    InternetCloseHandle(NetHandle);
  end
  else // NetHandle is not valid -> raise an exception
    //raise Exception.Create('Unable to initialize Wininet!');
    Result := '';
end;

//
// extract web location (only keep location from url)
//
function ExtractLocation(webUrl: string):string;
var
  c: integer;
  t: string;
begin
  webUrl := trim(webUrl);
  ExtractLocation := webUrl;
  c := 0;
  t := '';

  if lowerCase(copy(webUrl, 1, 8)) = 'https://' then
  begin
    webUrl := copy(webUrl, 9, length(webUrl));
    t := t + 'https://';
  end;

  if lowerCase(copy(webUrl, 1, 7)) = 'http://' then
  begin
    webUrl := copy(webUrl, 8, length(webUrl));
    t := t + 'http://';
  end;

  if lowerCase(copy(webUrl, 1, 4)) = 'www.' then
  begin
    webUrl := copy(webUrl, 5, length(webUrl));
    t := t+'www.';
  end;

  repeat c := c + 1;
  if copy(webUrl, c, 1) = '/' then
    webUrl := copy(webUrl, 1, c - 1);
  until c >= length(webUrl);
  c := length(webUrl) - 1;

  if t <> '' then webUrl := t + webUrl;
  ExtractLocation := webUrl;
end;

function GetDBFileName:string;
begin
  Result := IncludeTrailingPathDelimiter(ExtractFilePath(Application.ExeName)) + 'seoeq.mdb';
end;

//
// populate the Combo Box with data from the database
//
// ado = a TADOTable to connect to
// table_name = the table name
// filter_column_name = the column used as a filter
// setting_name = what record to extract (filter value)
// res_column_name = what value to add to the combo box
// cb = the combo box to populate
//
function PopulateCB(table_name, filter_column_name, setting_name, res_column_name: string; cb: TComboBox):integer;
var
  ado: TADOTable;
begin
  ado := TADOTable.Create(nil);
  Result := 0;
  cb.Items.Clear;
  cb.Text := '';
  with ado do
  begin
    ConnectionString := 'Provider=Microsoft.Jet.OLEDB.4.0;Data Source=' + GetDBFileName() + ';Persist Security Info=False';
    TableName := table_name;
    Active := True;
    Filter := filter_column_name + ' = '+ QuotedStr(setting_name);
    Filtered := True;
    First;
    while not EOF do
    begin
      cb.Items.Add(FieldByName(res_column_name).AsString);
      cb.ItemIndex := 0;
      Next;
      Result := Result + 1;
    end;

    // make sure the table is not filtered anymore
    Filter := '';
    Filtered := True;
    Filtered := False;
    Close;
    Active := False;
  end;
  ado.Free;
end;

//
// function to decode an url
//
function ASCIIURLDecode(Link: string):string;
var
  cnt:  integer;
begin
  for cnt := 32 to 125 do
  begin
    Link := StringReplace(Link, '%' + LowerCase(IntToHex(Cnt, 2)) + '', Chr(Cnt), [rfReplaceAll]);
    Link := StringReplace(Link, '%' + UpperCase(IntToHex(Cnt, 2)) + '', Chr(Cnt), [rfReplaceAll]);
  end;
  Result := Link;
end;

//
// function to get the Google PageRank for aURL
//
function GetGooglePageRank(aURL: string):string;
var
  url,
  url2,
  final   : string;
  i,
  len,
  counter : integer;
  AX,
  BX,
  CX,
  DX,
  DI,
  SI      : LongWord;
begin
  if (Pos('?', url) > 0) then Delete(url, Pos('?', url), Length(url)); // remove the parameters from the url, if any
  
	url := 'info:' + aURL;
  If Pos('?', url) > 0 then Delete(url, 1, Pos('?', url));
  If Pos('&', url) > 0 then Delete(url, 1, Pos('&', url));

	url2 := url;
  { Change all occurences of %XX to valid characters... }
	i := Pos('%', url);
  if i > 0 then url := ASCIIURLDecode(url);

	len := Length(url);
	DI := $9E3779B9;
	SI := $E6359A60;

	BX := DI;
	DX := 0;
	CX := 0;

	if (len > 12) then
  begin
		counter := len div 12;

		while (counter > 0) do begin
			DI := LongWord(ord(url[CX +  8])) shl{<<} 24 +
            LongWord(ord(url[CX +  7])) shl{<<} 16 +
            LongWord(ord(url[CX +  6])) shl{<<}  8 +
            LongWord(ord(url[CX +  5]))            + DI;

			SI := LongWord(ord(url[CX + 12])) shl{<<} 24 +
            LongWord(ord(url[CX + 11])) shl{<<} 16 +
            LongWord(ord(url[CX + 10])) shl{<<}  8 +
            LongWord(ord(url[CX +  9]))            + SI;

			DX := LongWord(ord(url[CX +  4])) shl{<<} 24 +
            LongWord(ord(url[CX +  3])) shl{<<} 16 +
            LongWord(ord(url[CX +  2])) shl{<<}  8 +
            LongWord(ord(url[CX +  1]))            + BX;

			DX := (DX - DI - SI)  xor (SI shr{>>} 13);
			DI := (DI - DX - SI)  xor (DX shl{<<}  8);
			SI := (SI - DI - DX)  xor (DI shr{>>} 13);

			DX := (DX - DI - SI)  xor (SI shr{>>} 12);
			DI := (DI - DX - SI)  xor (DX shl{<<} 16);
			SI := (SI - DI - DX)  xor (DI shr{>>}  5);

			DX := (DX - DI - SI)  xor (SI shr{>>}  3);
			DI := (DI - DX - SI)  xor (DX shl{<<} 10);
			SI := (SI - DI - DX)  xor (DI shr{>>} 15);
			BX :=  DX;

			Dec(counter);
			CX := CX + 12;
		end;
	end;

	SI := SI + len;
	AX := len - CX;

	If (AX > 0) then begin
		If (AX = 11) then
      SI := SI + LongWord(ord(url[CX + 11])) shl{<<} 24;
		If (AX >= 10) then
      SI := SI + LongWord(ord(url[CX + 10])) shl{<<} 16;
		If (AX >=  9) then
      SI := SI + LongWord(ord(url[CX +  9])) shl{<<}  8;
		If (AX >=  8) then begin
      DI := DI + LongWord(ord(url[CX +  8])) shl{<<} 24 +
      	         LongWord(ord(url[CX +  7])) shl{<<} 16 +
			   			   LongWord(ord(url[CX +  6])) shl{<<}  8 +
						     LongWord(ord(url[CX +  5]));
    end
		else begin
			If (AX >=  7) then
        DI := DI + LongWord(ord(url[CX +  7])) shl{<<} 16;
			If (AX >=  6) then
        DI := DI + LongWord(ord(url[CX +  6])) shl{<<}  8;
			If (AX >=  5) then
        DI := DI + LongWord(ord(url[CX +  5]));
		end;

	  If (AX >=  4) then begin
      BX := BX + LongWord(ord(url[CX +  4])) shl{<<} 24 +
                 LongWord(ord(url[CX +  3])) shl{<<} 16 +
                 LongWord(ord(url[CX +  2])) shl{<<}  8 +
                 LongWord(ord(url[CX +  1]));
    end
		else begin
			If (AX >=  3) then
        BX := BX + LongWord(ord(url[CX +  3])) shl{<<} 16;
			If (AX >=  2) then
        BX := BX + LongWord(ord(url[CX +  2])) shl{<<}  8;
			If (AX >=  1) then
        BX := BX + LongWord(ord(url[CX +  1]));
		end;
	end;

	BX := (BX - DI - SI)  xor (SI shr{>>} 13);
	DI := (DI - BX - SI)  xor (BX shl{<<}  8);
	SI := (SI - DI - BX)  xor (DI shr{>>} 13);

	BX := (BX - DI - SI)  xor (SI shr{>>} 12);
	DI := (DI - BX - SI)  xor (BX shl{<<} 16);
	SI := (SI - DI - BX)  xor (DI shr{>>}  5);

	AX := SI;
	BX := (BX - DI - AX)  xor (AX shr{>>}  3);
	DI := (DI - BX - AX)  xor (BX shl{<<} 10);
	AX := (AX - DI - BX)  xor (DI shr{>>} 15);

  final := Trim(DownloadFile3('http://toolbarqueries.google.com/search?client=navclient-auto&ch=6' + IntToStr(AX)+ '&features=Rank&q=' + url2 + ''));

  // Result := Copy(final, Length(final), 1);  (pr can be 10, look at google.com, so just the last char isn't enough)

  while (Pos(':', final) > 0) do
  begin
    Delete(final, 1, Pos(':', final));
  end;
  Result := Trim(final);
  if (Trim(Result) = '')  then Result := '0'; // if we didn't get any PR, return it as PR 0
  if (Length(Result) > 2) then Result := '0'; // if we got more than 2 chars for the PR, return it as PR 0; most likely we got an error message from Google
end;

//
// function to get the Google PageRank for aURL (test)
//
function GetGooglePageRank2(aURL: string; var test: string):string;
var
  url,
  url2,
  final   : string;
  i,
  len,
  counter : integer;
  AX,
  BX,
  CX,
  DX,
  DI,
  SI      : LongWord;
begin
	url := 'info:' + aURL;
  If Pos('?', url) > 0 then Delete(url, 1, Pos('?', url));
  If Pos('&', url) > 0 then Delete(url, 1, Pos('&', url));

	url2 := url;
  { Change all occurences of %XX to valid characters... }
	i := Pos('%', url);
  if i > 0 then url := ASCIIURLDecode(url);

	len := Length(url);
	DI := $9E3779B9;
	SI := $E6359A60;

	BX := DI;
	DX := 0;
	CX := 0;

	if (len > 12) then
  begin
		counter := len div 12;

		while (counter > 0) do begin
			DI := LongWord(ord(url[CX +  8])) shl{<<} 24 +
            LongWord(ord(url[CX +  7])) shl{<<} 16 +
            LongWord(ord(url[CX +  6])) shl{<<}  8 +
            LongWord(ord(url[CX +  5]))            + DI;

			SI := LongWord(ord(url[CX + 12])) shl{<<} 24 +
            LongWord(ord(url[CX + 11])) shl{<<} 16 +
            LongWord(ord(url[CX + 10])) shl{<<}  8 +
            LongWord(ord(url[CX +  9]))            + SI;

			DX := LongWord(ord(url[CX +  4])) shl{<<} 24 +
            LongWord(ord(url[CX +  3])) shl{<<} 16 +
            LongWord(ord(url[CX +  2])) shl{<<}  8 +
            LongWord(ord(url[CX +  1]))            + BX;

			DX := (DX - DI - SI)  xor (SI shr{>>} 13);
			DI := (DI - DX - SI)  xor (DX shl{<<}  8);
			SI := (SI - DI - DX)  xor (DI shr{>>} 13);

			DX := (DX - DI - SI)  xor (SI shr{>>} 12);
			DI := (DI - DX - SI)  xor (DX shl{<<} 16);
			SI := (SI - DI - DX)  xor (DI shr{>>}  5);

			DX := (DX - DI - SI)  xor (SI shr{>>}  3);
			DI := (DI - DX - SI)  xor (DX shl{<<} 10);
			SI := (SI - DI - DX)  xor (DI shr{>>} 15);
			BX :=  DX;

			Dec(counter);
			CX := CX + 12;
		end;
	end;

	SI := SI + len;
	AX := len - CX;

	If (AX > 0) then begin
		If (AX = 11) then
      SI := SI + LongWord(ord(url[CX + 11])) shl{<<} 24;
		If (AX >= 10) then
      SI := SI + LongWord(ord(url[CX + 10])) shl{<<} 16;
		If (AX >=  9) then
      SI := SI + LongWord(ord(url[CX +  9])) shl{<<}  8;
		If (AX >=  8) then begin
      DI := DI + LongWord(ord(url[CX +  8])) shl{<<} 24 +
      	         LongWord(ord(url[CX +  7])) shl{<<} 16 +
			   			   LongWord(ord(url[CX +  6])) shl{<<}  8 +
						     LongWord(ord(url[CX +  5]));
    end
		else begin
			If (AX >=  7) then
        DI := DI + LongWord(ord(url[CX +  7])) shl{<<} 16;
			If (AX >=  6) then
        DI := DI + LongWord(ord(url[CX +  6])) shl{<<}  8;
			If (AX >=  5) then
        DI := DI + LongWord(ord(url[CX +  5]));
		end;

	  If (AX >=  4) then begin
      BX := BX + LongWord(ord(url[CX +  4])) shl{<<} 24 +
                 LongWord(ord(url[CX +  3])) shl{<<} 16 +
                 LongWord(ord(url[CX +  2])) shl{<<}  8 +
                 LongWord(ord(url[CX +  1]));
    end
		else begin
			If (AX >=  3) then
        BX := BX + LongWord(ord(url[CX +  3])) shl{<<} 16;
			If (AX >=  2) then
        BX := BX + LongWord(ord(url[CX +  2])) shl{<<}  8;
			If (AX >=  1) then
        BX := BX + LongWord(ord(url[CX +  1]));
		end;
	end;

	BX := (BX - DI - SI)  xor (SI shr{>>} 13);
	DI := (DI - BX - SI)  xor (BX shl{<<}  8);
	SI := (SI - DI - BX)  xor (DI shr{>>} 13);

	BX := (BX - DI - SI)  xor (SI shr{>>} 12);
	DI := (DI - BX - SI)  xor (BX shl{<<} 16);
	SI := (SI - DI - BX)  xor (DI shr{>>}  5);

	AX := SI;
	BX := (BX - DI - AX)  xor (AX shr{>>}  3);
	DI := (DI - BX - AX)  xor (BX shl{<<} 10);
	AX := (AX - DI - BX)  xor (DI shr{>>} 15);

  //final := Trim(GetPage('http://toolbarqueries.google.com/search?client=navclient-auto&ch=6' + IntToStr(AX)+ '&features=Rank&q=' + url2 + '', aUserAgent));
  final := Trim(DownloadFile3('http://toolbarqueries.google.com/search?client=navclient-auto&ch=6' + IntToStr(AX)+ '&features=Rank&q=' + url2 + ''));

  while (Pos(':', final) > 0) do
  begin
    Delete(final, 1, Pos(':', final));
  end;

  test := final;

  Result := Copy(final, Length(final), 1);
end;

//
// function to extract every folder in a folder tree
//
function PathToFolders(path: string; var folders: TStringList):integer;
var
  s:  string;
  t:  string;
  lc: char;
  j:  integer;
begin
  if RightStr(path, 1) <> '\' then t := path + '\'
  else t := path;

  lc := ' '; j := 1; s := '';
  while j <= Length(t) do
  begin
    if (t[j] <> '\') or (lc <> '\') then s := s + t[j];
    lc := t[j];
    j := j + 1;
  end;

  while Length(s) > 0 do
  begin
    t := Copy(s, 1, Pos('\', s));
    Delete(s, 1, Pos('\', s));
    if folders.Count <= 0 then
      folders.Add(t)
    else
      folders.Add(folders[folders.count - 1] + t);
  end;
  folders.Delete(0);
  Result := folders.Count;
end;

//
// function to check if all folders in a path exists; if create = true, create path
//
function CheckPath(folders: TStringList; create: boolean):boolean;
var
  j:  integer;
begin
  for j := 0 to folders.Count - 1 do
    if not DirectoryExists(folders[j]) then
    begin
      if create then
        Result := CreateDir(folders[j])
      else
      begin
        Result := False;
        Break;
      end;
    end
    else Result := True;
end;

//
// PathToFolders + CheckPath
//
function CheckFolder(path: string; create: boolean):boolean;
var
  sl: TStringList;
begin
  sl  := TStringList.Create;
  PathToFolders(path, sl);
  Result := CheckPath(sl, create);
  sl.Clear;
  sl.Free;
end;

//
// replace special chars in path (:, *, /, \, ?, ....) with -
//
function NormalizeFilename(path: string):string;
var
  s: string;
begin
  s := path;
  s := FastReplace(LowerCase(s), ' ', '-');
  s := FastReplace(LowerCase(s), '?', '-');
  s := FastReplace(LowerCase(s), '*', '-');
  s := FastReplace(LowerCase(s), '/', '-');
  s := FastReplace(LowerCase(s), '|', '-');
  s := FastReplace(LowerCase(s), ':', '-');
  s := FastReplace(LowerCase(s), '"', '-');
  s := FastReplace(LowerCase(s), '>', '-');
  s := FastReplace(LowerCase(s), '<', '-');
  Result := s;
end;

//
// replace special chars in path (:, *, ?, ....) with -
//
function NormalizeFilename2(path: string):string;
var
  s: string;
begin
  s := path;
  s := FastReplace(LowerCase(s), ' ', '-');
  s := FastReplace(LowerCase(s), '?', '-');
  s := FastReplace(LowerCase(s), '*', '-');
  s := FastReplace(LowerCase(s), '|', '-');
  s := FastReplace(LowerCase(s), ':', '-');
  s := FastReplace(LowerCase(s), '"', '-');
  s := FastReplace(LowerCase(s), '>', '-');
  s := FastReplace(LowerCase(s), '<', '-');
  Result := s;
end;

//
// function to remove a folder and its contents
//
function DelTree(DirName: string): boolean;
var
  SHFileOpStruct: TSHFileOpStruct;
  DirBuf: array [0..255] of char;
begin
  try
   FillChar(SHFileOpStruct,Sizeof(SHFileOpStruct),0);
   FillChar(DirBuf, Sizeof(DirBuf), 0);
   StrPCopy(DirBuf, DirName);
   with SHFileOpStruct do begin
    Wnd := 0;
    pFrom := @DirBuf;
    wFunc := FO_DELETE;
    fFlags := FOF_ALLOWUNDO;
    fFlags := fFlags or FOF_NOCONFIRMATION;
    fFlags := fFlags or FOF_SILENT;
   end;
    Result := (SHFileOperation(SHFileOpStruct) = 0);
   except
    Result := False;
  end;
end;

//
// extract domain name (only keep domain from url)
//
function ExtractDomain(webUrl: string):string;
var
  c: integer;
  t: string;
begin
  webUrl := trim(webUrl);
  ExtractDomain := webUrl;
  c := 0;
  t := '';

  if lowerCase(copy(webUrl, 1, 8)) = 'https://' then
  begin
    webUrl := copy(webUrl, 9, length(webUrl));
  end;

  if lowerCase(copy(webUrl, 1, 7)) = 'http://' then
  begin
    webUrl := copy(webUrl, 8, length(webUrl));
  end;

  if lowerCase(copy(webUrl, 1, 4)) = 'www.' then
  begin
    webUrl := copy(webUrl, 5, length(webUrl));
  end;

  repeat c := c + 1;
  if copy(webUrl, c, 1) = '/' then
    webUrl := copy(webUrl, 1, c - 1);
  until c >= length(webUrl);
  c := length(webUrl) - 1;

  if t <> '' then webUrl := t + webUrl;
  ExtractDomain := webUrl;
end;

//
// function to get the anchor text for a url
//
function GetAnchorText(html, url: string):string;
var
  i:  integer;
begin
  if RightStr(url, 1) = '/' then Delete(url, Length(url), 1);
  Result := '';
  i := Pos(url, html);
  if (i = 0) then Exit;
  Delete(html, 1, i + Length(url) - 1);
  i := Pos('>', html);
  if (i = 0) then Exit;
  Delete(html, 1, i);
  i := Pos('<', html);
  if (i = 0) then Exit;
  Delete(html, i, Length(html));
  Result := Trim(ExtractNonHTML(html));
end;

//
// function to find links inside an html file; do not extract links on the same 'domain'
//
procedure GetLinks(text: string; domain: string; var sl: TStringList);
var
  s: string;
  t: string;
begin
  with TRegExpr.Create do
  try
    Expression :=
     '(?i)'
     + '('
     + '(HTTP|HTTPS)://' // Protocol
     + '|www\.)' // trick to catch links without protocol (like 'www.paycash.ru')
     + '([\w\d\-]+(\.[\w\d\-]+)+)' // TCP addr / domain name
     + '(:\d\d?\d?\d?\d?)?' // port number
     + '(((/[%+\w\d\-\\\.]*)+)*)'; // unix path
    if Exec(text) then
      repeat
        begin
          s := Trim(Match[0]);
          // do not extract links on the same domain, only outbound links
          if (Pos(ExtractDomain(domain), s) > 0) then s := '';
          // if the link is in a form like this: "http://www.domain.com" then extract it
          if (s <> '') and (RightStr(s, Length(ExtractDomain(s))) = ExtractDomain(s)) then s := s + '/';
          // extract outbound links ending with /, .html, .htm, .php, .asp, .chm
          if  (Pos('.html', s) = 0) and
              (Pos('.htm', s) = 0)  and
              (Pos('.php', s) = 0)  and
              (Pos('.asp', s) = 0)  and
              (Pos('.chm', s) = 0)  and
              (RightStr(s, 1) <> '/') then s := '';
          if (s <> '') and (sl.IndexOf(s) < 0) then
          begin
            sl.Add(s);
            //Say(Format('Link found: %s', [s]));
          end;
        end;
      until not ExecNext;
    finally Free;
  end;
end;

function GetOutboundLinksCount(page, url: string):integer;
var
  sl: TStringList;
begin
  sl := TStringList.Create;
  GetLinks(page, url, sl);
  Result := sl.Count;
  sl.Clear;
  sl.Free;
end;

function GetAllLinksCount(page: string):integer;
var
  sl: TStringList;
begin
  sl := TStringList.Create;
  GetLinks(page, 'abcd1234zxcv', sl);
  Result := sl.Count;
  sl.Clear;
  sl.Free;
end;

function  GetIndexedPages(url: string):longint;
var
  html: string;
  link: string;
begin
  link := 'http://www.google.com/search?q=site:MY_URL&hl=en&filter=0&num=1';
  link := FastReplace(link, 'MY_URL', url, false);
  html := Trim(DownloadFile3(link));
  RemoveHead(html);
  if (Pos('Results', html) > 0) then Delete(html, 1, Pos('Results', html) + 7);
  if (Pos('from', html) > 0) then Delete(html, Pos('from', html), Length(html));
  if (Pos('of', html) > 0) then Delete(html, 1, Pos('of', html) + 2);
  html := Trim(html);
  if (Pos(' ', html) > 0) then Delete(html, 1, Pos(' ', html));
  html := ExtractNonHTML(html);
  html := Trim(FastReplace(html, ',', '', false));
  html := Trim(FastReplace(html, '.', '', false));
  try
    Result := StrToInt(html);
  except
    Result := 0;
  end;
end;


function UrlEncodeRFC3986(S: string): string;

  function _IntToHex(Value: Integer; Digits: Integer): String;
  begin
    Result := SysUtils.IntToHex(Value, Digits);
  end;

var
  I: Integer;
  Ch: Char;
begin
  Result := '';
  for I := 1 to Length(S) do begin
    Ch := S[I];
    if ((Ch >= '0') and (Ch <= '9')) or
    ((Ch >= 'a') and (Ch <= 'z')) or
    ((Ch >= 'A') and (Ch <= 'Z')) or
    (Ch = '.') or (Ch = '-') or (Ch = '_') or (Ch = '~')then
      Result := Result + Ch
    else
      Result := Result + '%' + _IntToHex(Ord(Ch), 2);
  end;
end;

function Say(what: string):boolean;
begin
  OutputDebugString(PChar('SEO Equalizer: ' + what));
end;

end.
